import sys 
sys.path.append("..")

import unittest 
import uuid
import datetime 
import logging
import botocore
from loris_log.boto3Client import Boto3Client
from loris_log.customLog import CustomLog
from loris_log.ftpCllient import FtpClient
from loris_log.myException import InvalidFTPHostNameException, InvalidFTPPortNumberException,\
    InvalidMessageException, InvalidFTPUserNameException, InvalidRoleSecretException, \
    InvalidRoleAccessKeyException, InvalidFTPPasswordException, EmptyParameterException,\
    NoneValueException, InvalidAttributeException, FTPConnectionFailedException,\
    FTPFileCreationException, InvalidRegionNameException

        
class TestBoto3Client(unittest.TestCase):
    
    def test_boto3_client_with_empty_region_name(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        
        with self.assertRaises(InvalidRegionNameException) as context:
            Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        self.assertTrue("Invalid region name." in str(context.exception))
        
    def test_boto3_client_with_empty_access_key(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        
        with self.assertRaises(InvalidRoleAccessKeyException) as context:
            Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        self.assertTrue("Invalid role access key." in str(context.exception))
        
    def test_boto3_client_with_empty_secret(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        
        with self.assertRaises(InvalidRoleSecretException) as context:
            Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        self.assertTrue("Invalid role secret." in str(context.exception))
        
    def test_get_log_group(self):
        target_region_name = ""
        target_aws_key = "s"
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        response = boto3Client.get_log_groups()
        self.assertTrue(len(response) > 0)
        
    def test_get_log_stream(self):
        target_region_name = ""
        target_aws_key = "s"
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        response = boto3Client.get_log_stream()
        self.assertTrue(len(response) > 0)
        
    def test_boto3_client_with_other_attribute_secret(self):
        target_region_name = ""
        target_aws_key = 123
        target_aws_secret = ""
        with self.assertRaises(InvalidAttributeException) as context:
            Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_boto3_client_with_wrong_password(self):
        target_region_name = ""
        target_aws_key = "s"
        target_aws_secret = "123412"
        # with self.assertRaises(AWSConnectionException) as context:
        client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        # self.assertTrue("AWS connection failure." in str(context.exception))

    def test_get_log_group(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)

        response = boto3Client.get_log_groups()
        self.assertIsNot(len(response), 0)
        
    def test_get_log_stream(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        target_log_group = "sample"
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        response = boto3Client.get_log_stream(target_log_group)
        self.assertIsNotNone(response)
    
    def test_create_log_groups_stream_again(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample"
        target_log_stream = "sample-log"
        creation_response = boto3Client.create_log_group_stream(target_log_group, target_log_stream)
        self.assertIn(creation_response, [True, False])
        
        target_message = "target sample message"
        response = boto3Client.set_log_message(target_log_group, target_log_stream, target_message)
        self.assertIn(response, [True, False])
        
    def test_create_log_stream_with_empty_log_group(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample"
        target_log_stream = "sample-log"
        with self.assertRaises(EmptyParameterException) as context:
            creation_response = boto3Client.create_log_group_stream("", target_log_stream)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_create_log_stream_with_other_attribute_log_stream_name(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample"
        target_log_stream = "sample-log"
        with self.assertRaises(InvalidAttributeException) as context:
            creation_response = boto3Client.create_log_group_stream(target_log_group, None)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_set_log_message_on_not_exist_log_group(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample1"
        target_log_stream = "sample-log"
        target_message = "target sample message"
        response = boto3Client.set_log_message(target_log_group, target_log_stream, target_message)
        self.assertFalse(response)
        
    def test_set_log_with_wrong_attribute(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = None
        target_log_stream = "sample-log"
        target_message = "target sample message"
        
        with self.assertRaises(InvalidAttributeException) as context:
            response = boto3Client.set_log_message(target_log_group, target_log_stream, target_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_set_log_with_empty_message(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample"
        target_log_stream = "sample-log"
        target_message = ""
        
        with self.assertRaises(EmptyParameterException) as context:
            response = boto3Client.set_log_message(target_log_group, target_log_stream, target_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_create_log_groups_stream(self):
        target_region_name = ""
        target_aws_key = ""
        target_aws_secret = ""
        boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
        target_log_group = "sample"
        target_log_stream = "sample-log"
        creation_response = boto3Client.create_log_group_stream(target_log_group, target_log_stream)
        self.assertIn(creation_response, [True, False])
        
        target_message = "target sample message"
        response = boto3Client.set_log_message(target_log_group, target_log_stream, target_message)
        self.assertIn(response, [True, False])
        
class TestCustomLog(unittest.TestCase):
    
    def test_error_log_message(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = uuid.uuid4()
        
        error_message = CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertNotEqual(len(error_message), 0)
        
    def test_error_empty_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = ""
        id = uuid.uuid4()
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_error_number_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = 123.1938484
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_error_log_none_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = None
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_error_log_NaN_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_error_log_incomplete_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
        
    def test_error_log_empty_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = ""
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_error_log_empty_message(self):
        target_error_message = [""]
        target_name = "app"
        id = uuid.uuid4()
    
        
        error_message = CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue(len(error_message) > 0)
        
    def test_error_log_empty_list_message(self):
        target_error_message = []
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_error_log_empty_list_message(self):
        target_error_message = None
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_info_log_empty_message_weird_message(self):
        target_error_message = [None, 123]
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_fatal_log_empty_message_weird_message(self):
        target_error_message = [None, 123]
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_debug_log_empty_message_weird_message(self):
        target_error_message = [None, 123]
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_error_log_empty_message_weird_message(self):
        target_error_message = [None, 123]
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_error_log_number_message(self):
        target_error_message = [1, 2, 3]
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_error_log_number_message(self):
        target_error_message = 12
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_error_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
    # #########################################################################################
    # TEST set_debug_message function
    def test_debug_log_message(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = uuid.uuid4()
        
        debug_message = CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertNotEqual(len(debug_message), 0)
        
    def test_debug_empty_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = ""
        id = uuid.uuid4()
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_debug_number_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = 123.1938484
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_debug_log_none_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = None
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_debug_log_NaN_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_error_debug_incomplete_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
        
    def test_debug_log_empty_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = ""
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_debug_log_empty_message(self):
        target_error_message = [""]
        target_name = "app"
        id = uuid.uuid4()
    
        
        error_message = CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue(len(error_message) > 0)
        
    def test_debug_log_empty_list_message(self):
        target_error_message = []
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_debug_log_empty_list_message(self):
        target_error_message = None
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_debug_log_number_message(self):
        target_error_message = [1, 2, 3]
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_debug_log_number_message(self):
        target_error_message = 12
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_debug_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
    
    # ##########################################################################################
    # Test set_fatal_message function
    
    def test_fatal_log_message(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = uuid.uuid4()
        
        fatal_message = CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertNotEqual(len(fatal_message), 0)
        
    def test_fatal_empty_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = ""
        id = uuid.uuid4()
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_fatal_number_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = 123.1938484
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_fatal_log_none_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = None
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_fatal_log_NaN_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_fatal_debug_incomplete_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
        
    def test_fatal_log_empty_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = ""
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_fatal_log_empty_message(self):
        target_error_message = [""]
        target_name = "app"
        id = uuid.uuid4()
    
        
        error_message = CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue(len(error_message) > 0)
        
    def test_fatal_log_empty_list_message(self):
        target_error_message = []
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_fatal_log_empty_list_message(self):
        target_error_message = None
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_fatal_log_number_message(self):
        target_error_message = [1, 2, 3]
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_fatal_log_number_message(self):
        target_error_message = 12
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    # ######################################################################################
    # Test set_info_message function
    
    def test_info_log_message(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = uuid.uuid4()
        
        info_message = CustomLog.set_fatal_message(id, target_name, target_error_message)
        self.assertNotEqual(len(info_message), 0)
        
    def test_info_empty_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = ""
        id = uuid.uuid4()
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_info_number_name(self):
        target_error_message = ["system failed", "exception"]
        target_name = 123.1938484
        id = uuid.uuid4()
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_info_log_none_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = None
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_info_log_NaN_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_info_log_incomplete_uuid(self):
        import math
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = math.nan
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
        
    def test_info_log_empty_uuid(self):
        target_error_message = ["system failed", "exception"]
        target_name = "app"
        id = ""
        
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_info_log_empty_message(self):
        target_error_message = [""]
        target_name = "app"
        id = uuid.uuid4()
    
        
        error_message = CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue(len(error_message) > 0)
        
    def test_info_log_empty_list_message(self):
        target_error_message = []
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_info_log_empty_list_message(self):
        target_error_message = None
        target_name = "app"
        id = uuid.uuid4()
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    
        
    def test_info_log_number_message(self):
        target_error_message = [1, 2, 3]
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_info_log_number_message(self):
        target_error_message = 12
        target_name = "app"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_info_message(id, target_name, target_error_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    # ######################################################################################
    # Test set_ftp_log_message function 
    
    def test_ftp_log_message(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        file_data_message = CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertIn(target_result, file_data_message)
        self.assertIn(target_ground_truth, file_data_message)
        
    def test_frp_log_message_other_attribute_result(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"7890-cm,"
        target_ground_truth = b"12"
        id = uuid.uuid4()
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Invalid message." in str(context.exception))
        
    def test_ftp_log_message_other_attribute_groundtruth(self):
        import math
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"wwertyuio"
        id = uuid.uuid4()
        with self.assertRaises(InvalidMessageException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Invalid message." in str(context.exception))
        
        
    def test_ftp_log_message_empty_groundtruth(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = ""
        id = uuid.uuid4()
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_ftp_log_message_none_groundtruth(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = None
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_ftp_log_message_empty_result(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = ""
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_ftp_log_message_other_attribute_end_time(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = []
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attribute error." in str(context.exception))
        
    
    def test_ftp_log_message_empty_end_time(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = ""
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_ftp_log_message_none_end_time(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = None
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_ftp_log_message_wrong_start_time(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_ftp_log_message_empty_start_time(self):
        start_time = datetime.datetime.now()
        target_start_time = ""
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(EmptyParameterException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_ftp_log_message_none_start_time(self):
        target_start_time = None
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_ftp_log_message_other_attribute_start_time(self):
        target_start_time = []
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = uuid.uuid4()
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_ftp_log_message_none_uuid(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = None
        
        with self.assertRaises(NoneValueException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_ftp_log_message_empty_uuid(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = ""
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_ftp_log_message_other_uuid(self):
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        id = [9, 8]
        
        with self.assertRaises(InvalidAttributeException) as context:
            CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
        self.assertTrue("Attribute error." in str(context.exception))
        
# #####################################################################################
class TestFtpClient(unittest.TestCase):
    
    def test_ftp_push_log_with_none_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message."])
        with self.assertRaises(NoneValueException) as context:
            response = client.set_ftp_log_file(None, 
                                    target_filename,
                                    log)
        self.assertTrue("The input value should not be none." in str(context.exception))
        
    def test_ftp_push_log_with_wrong_empty_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message."])
        with self.assertRaises(EmptyParameterException) as context:
            response = client.set_ftp_log_file(target_path, 
                                    "",
                                    log)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
        
    def test_ftp_push_log_with_wrong_attribute_log(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message."])
        with self.assertRaises(InvalidAttributeException) as context:
            response = client.set_ftp_log_file(target_path, 
                                    target_filename,
                                    123)
        self.assertTrue("Attribute error." in str(context.exception))
    
    def test_ftp_push_log(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message."])
        response = client.set_ftp_log_file(target_path, 
                                target_filename,
                                log)
        self.assertTrue(response)
        
    def test_ftp_push_log_same_directory_again(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message again."])
        response = client.set_ftp_log_file(target_path, 
                                target_filename,
                                log)
        self.assertTrue(response)
        
    def test_ftp_push_log_new_directory(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example_new"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_file(target_path, target_filename)
        customLog = CustomLog()
        ids = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
        log = customLog.set_debug_message(ids, "name", ["The sample debug message."])
        response = client.set_ftp_log_file(target_path, 
                                target_filename,
                                log)
        self.assertTrue(response)
        
    def test_ftp_push_log_data_with_other_attribute_data(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "/log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(LogWritingException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("Writing log failure." in str(context.exception))
    
    def test_ftp_push_log_data_with_other_attribute_data(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(InvalidAttributeException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    12234)
        self.assertTrue("Attribute error." in str(context.exception))
    
    def test_ftp_push_log_data_with_none_data(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(NoneValueException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    None)
        self.assertTrue("The input value should not be none." in str(context.exception))
    
    def test_ftp_push_log_data_with_empty_data(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(EmptyParameterException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    b"")
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
    
    def test_ftp_push_log_data_with_other_attribute_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = 100344
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(InvalidAttributeException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("Attribute error." in str(context.exception))
        
    def test_ftp_push_log_data_with_none_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = None
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(NoneValueException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
    
    def test_ftp_push_log_data_with_empty_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(EmptyParameterException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
    
    def test_ftp_push_log_data_with_other_attribute_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = 123445
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(InvalidAttributeException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("Attribute error." in str(context.exception))
    
    def test_ftp_push_log_data_with_none_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = None
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(NoneValueException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("The input value should not be none." in str(context.exception))
    
    def test_ftp_push_log_data_with_empty_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = ""
        target_filename = "2023-09-06_sample"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        with self.assertRaises(EmptyParameterException) as context:
            response = client.set_ftp_log_data(target_path, target_filename,
                                    data_message)
        self.assertTrue("Attempt to enter empty input." in str(context.exception))
    
    def test_ftp_push_log_data(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        target_path = "log_testing/example"
        target_filename = "ABCD-2345-CDFR"
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        client.create_ftp_log_data(target_path, target_filename)
        
        customLog = CustomLog()
        id = uuid.uuid4()
        start_time = datetime.datetime.now()
        target_start_time = start_time.strftime("%H:%M:%S")
        end_time = datetime.datetime.now()
        target_end_time = end_time.strftime("%H:%M:%S")
        target_result = b"32"
        target_ground_truth = b"3"
        data_message = customLog.set_ftp_log_data(
            id, 
            target_start_time,
            target_end_time,
            target_result,
            target_ground_truth
        )
        
        response = client.set_ftp_log_data(target_path, target_filename,
                                data_message)
        self.assertTrue(response)
    
    def test_ftp_log_file_creation_with_other_none_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "example/sa"
        target_filename = None
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_other_attribute_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "example/sa"
        target_filename = []
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_empty_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "example/sa"
        target_filename = ""
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_other_attribute_file_path2(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = 1234
        target_filename = "sample"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_other_attribute_file_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = []
        target_filename = "sample"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_none_file_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = None
        target_filename = "sample"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation_with_invalid_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "/log_testing/testing"
        target_filename = "sample"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_log_file_creation(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/testing"
        target_filename = "sample"
        response = client.create_ftp_log_file(target_path, target_filename)
        self.assertIn(response, [True, False])
    
    def test_ftp_file_server_creation_with_other_attribute_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/testing"
        target_filename = []
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation_with_none_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/testing"
        target_filename = None
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation_with_empty_filename(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/testing"
        target_filename = ""
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation_with_empty_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = ""
        target_filename = "ABCD_12345_EDFT"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation_with_none_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = None
        target_filename = "ABCD_12345_EDFT"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation_with_invalid_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "/log_testing/log"
        target_filename = "ABCD_12345_EDFT"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
    
    def test_ftp_file_server_creation(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/log"
        target_filename = "ABCD_12345_EDFT"
        response = client.create_ftp_log_data(target_path, target_filename)
        self.assertIn(response, [True, False, None])
    
    def test_ftp_log_file_creation_with_wrong_file_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = 1234
        target_filename = "ABCD_12345_EDFT"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_file(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
        
    def test_ftp_file_creation_with_wrong_file_path(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = 1234
        target_filename = "ABCD_12345_EDFT"
        with self.assertRaises(FTPFileCreationException) as context:
            client.create_ftp_log_data(target_path, target_filename)
        self.assertTrue("File or folder creation error." in str(context.exception))
        
    def test_ftp_file_server_creation_again(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        target_path = "log_testing/log"
        target_filename = "ABCD_12345_EDFT"
        response = client.create_ftp_log_data(target_path, target_filename)
        self.assertIn(response, [True, False, None])
    
    def test_ftp_connection(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        client = FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertIsNotNone(client)
    
    def test_ftp_connection_with_wrong_username(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = None
        with self.assertRaises(FTPConnectionFailedException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("FTP Server Connection Failure." in str(context.exception))    
    
    def test_ftp_connection_with_wrong_username(self):
        target_hostname = ""
        target_port = 00
        target_username = None
        target_password = "JtP$dsds887hP"
        with self.assertRaises(InvalidFTPUserNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Username." in str(context.exception))
        
    def test_ftp_connection_with_num_password(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = 876
        with self.assertRaises(InvalidFTPPasswordException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Password." in str(context.exception))
        
    def test_ftp_connection_with_wrong_password(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = "sdfg"
        with self.assertRaises(FTPConnectionFailedException) as context:
            FtpClient(target_hostname, target_port,\
                target_username, target_password)
        self.assertTrue("FTP Server Connection Failure." in str(context.exception))
        
    def test_ftp_connection_with_none_username(self):
        target_hostname = ""
        target_port = 00
        target_username = None
        target_password = ""
        with self.assertRaises(InvalidFTPUserNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Username." in str(context.exception))
        
    def test_ftp_connection_with_none_port(self):
        target_hostname = ""
        target_port = None
        target_username = ""
        target_password = ""
        with self.assertRaises(InvalidFTPPortNumberException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Port Number." in str(context.exception)) 
        
    def test_fto_connection_wrong_port(self):
        target_hostname = ""
        target_port = 290
        target_username = ""
        target_password = ""
        with self.assertRaises(FTPConnectionFailedException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("FTP Server Connection Failure." in str(context.exception)) 
    
    def test_ftp_connection_with_wrong_hostname(self):
        target_hostname = "1.2.3.323"
        target_port = 00
        target_username = ""
        target_password = ""
        with self.assertRaises(InvalidFTPHostNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Host name." in str(context.exception))    
    
    def test_ftp_connection_with_other_attribute_hostname(self):
        target_hostname = []
        target_port = 00
        target_username = ""
        target_password = ""
        with self.assertRaises(InvalidFTPHostNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Host name." in str(context.exception))
    
    def test_ftp_connection_with_none_hostname(self):
        target_hostname = None
        target_port = 00
        target_username = ""
        target_password = ""
        with self.assertRaises(InvalidFTPHostNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Host name." in str(context.exception))
        
    def test_ftp_connection_with_empty_hostname(self):
        target_hostname = ""
        target_port = 00
        target_username = ""
        target_password = ""
        with self.assertRaises(InvalidFTPHostNameException) as context:
            FtpClient(target_hostname, target_port,\
            target_username, target_password)
        self.assertTrue("Invalid FTP Host name." in str(context.exception))
        
if __name__ == "__main__":
    logging.basicConfig(stream=sys.stderr)
    logging.getLogger("logTest").setLevel(logging.DEBUG)
    unittest.main() 


"""This library own customized exception.
"""

class InvalidFTPHostNameException(Exception):
    """
    Custom exception for the invalid FTP hostname.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid FTP Host name.")

class InvalidFTPPortNumberException(Exception):
    """
    Custom exception for the invalid FTP port number.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid FTP Port Number.")

class InvalidFTPUserNameException(Exception):
    """
    Custom exceptionf or the invalid FTP username.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid FTP Username.")

class InvalidFTPPasswordException(Exception):
    """
    Custom exception for the invalid FTP password.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid FTP Password.")

class InvalidMessageException(Exception):
    """
    Custom Exception for the invalid message.
    
    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid message.")

class InvalidRoleSecretException(Exception):
    """
    Custom exception for the invalid role secret.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid role secret.")

class EmptyParameterException(Exception):
    """
    Custom exception for the empty parameter.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Attempt to enter empty input.")

class InvalidRoleAccessKeyException(Exception):
    """
    Custom exception for the invalid role access key.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Invalid role access key.")

class NoneValueException(Exception):
    """Custom exception for the non value.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("The input value should not be none.")

class InvalidAttributeException(Exception):
    """Custom exception for attribute error.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("Attribute error.")

class InvalidRegionNameException(Exception):
    """Custom exception for invalid region name.

    Args:
        Exception (string): Invalid region name.
    """
    def __str__(self):
        return repr("Invalid region name.")

class FTPConnectionFailedException(Exception):
    """
    Custom exception for FTP Connection Failure.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("FTP Server Connection Failure.")

class FTPFileCreationException(Exception):
    """
    Custom exception for file or folder creation failure.

    Args:
        Exception (string): The error message.
    """
    def __str__(self):
        return repr("File or folder creation error.")



"""
The library to push log message to FTP server.
"""

import io
import datetime
import ipaddress
from ftplib import FTP, all_errors
from .myException import FTPConnectionFailedException,\
    FTPFileCreationException,\
    NoneValueException,\
    InvalidAttributeException,\
    EmptyParameterException,\
    InvalidFTPHostNameException,\
    InvalidFTPPortNumberException,\
    InvalidFTPUserNameException,\
    InvalidFTPPasswordException

# pylint: disable=E0302
# pylint: disable=W0212
class FtpClient:
    """Class to establish connection and writing to FTP server.
    """

    def __init__(self, hostname, port_num, username, password):
        """
        Establish communication with the remote FTP server.

        Args:
            hostname (string): FTP server hostname.
            port_num (string): FTP server port number.
            username (string): FTP server username.
            password (string): FTP server password.

        Raises:
            error: The FTP server connection establishment error.
        """

        if isinstance(port_num, int) is False:
            raise InvalidFTPPortNumberException()

        if isinstance(username, str) is False\
            or username is None or\
            username == "":
            raise InvalidFTPUserNameException()

        if isinstance(password, str) is False\
            or password is None or\
            password == "":
            raise InvalidFTPPasswordException()

        self.ftp_client = FTP()

        try:
            ipaddress.ip_address(hostname)
            self.ftp_client.connect(hostname, port_num)
            self.ftp_client.login(username, password)
        except all_errors as exc:
            raise FTPConnectionFailedException() from exc
        except ValueError as valuerr:
            raise InvalidFTPHostNameException() from valuerr

    def __get_ftp_directories(self):
        """
        To retrieve all the directories and files that are present 
        in the current working directory inside this directory 
        in this FTP server.

        Returns:
            list: A complete list of directories and files that are 
                currently available in present working directory.
        """
        dir_list = []
        self.ftp_client.retrlines('NLST', dir_list.append)
        return [item.split(" ")[-1] for item in dir_list]

    def __get_datetime(self):
        """
        Get the today date.

        Returns:
            string: Today's date.
        """
        return datetime.datetime.now().strftime("%Y-%m-%d")

    def __set_data_file_name(self, file_name):
        """
        Set the complete filename of the desired csv file.

        Args:
            file_name (string): The name of the csv file that wanted
                to be named.

        Returns:
            string: The complete filename of a csv file.
        """
        return self.__get_datetime() + "_"  + str(file_name) + ".csv"

    def __set_log_file_name(self, file_name):
        """
        Get the complete filename of the desired log file.

        Args:
            file_name (string): The name of the log file that wanted
                to be named.

        Returns:
            string: The complete fiilename of a log file.
        """
        return self.__get_datetime() + "_"  + str(file_name) + ".log"

    def create_ftp_log_data(self, directory_path, file_name):
        """
        Create a csv log file inside a FTP server based on the 
        predefined path.

        Args:
            directory_path (string): The path to the csv log file.
            file_name (string): Filename of the csv log file.

        Returns:
            boolean: True if the csv log file was successfully created;
                False otherwise.
        """
        if file_name == "" or file_name is None or\
            isinstance(file_name, str) is False:
            raise FTPFileCreationException()

        if directory_path == "" or directory_path is None or\
            isinstance(directory_path, str) is False:
            raise FTPFileCreationException()

        try:
            # get all the directory names
            directories = directory_path.split('/')
            for directory in directories:
                # check if the directory name is present
                if directory not in self.__get_ftp_directories():
                    # if not present create the directory
                    # then move to the corresponding directory
                    self.ftp_client.mkd(directory)
                    self.ftp_client.cwd(directory)
                else:
                    # if already present,
                    # just switch to the corresponding directory
                    self.ftp_client.cwd(directory)

            # Then create the desired log file if the log file
            # was not present before. Otherwise done nothing.
            filename = self.__set_data_file_name(file_name)
            if filename not in self.__get_ftp_directories():
                buf = io.BytesIO()
                buf.write(b"uuid,start,end,result,groundtruth\n")
                buf.seek(0)
                self.ftp_client.storbinary(f"STOR {filename}", buf)
        except all_errors as error:
            raise FTPFileCreationException() from error

    def set_ftp_log_data(self, directory_path, file_name, log_data):
        """
        Set the csv log file with the desired log data.

        Args:
            directory_path (string): The path to the csv log file.
            file_name (string): The name of the csv log file.
            log_data (string): The data that wanted to be logged.

        Returns:
            boolean: True if data was successfully logged; False otherwise.
        """
        if directory_path is None or file_name is None or\
            log_data is None:
            raise NoneValueException()

        if isinstance(directory_path, str) is False or\
            isinstance(file_name, str) is False or\
            isinstance(log_data, bytes) is False:
            raise InvalidAttributeException()

        if len(directory_path) == 0 or len(file_name) == 0 or\
            log_data == b"":
            raise EmptyParameterException()

        # get the desired directory name
        directories = directory_path.split('/')
        # loop through all the directory name
        for directory in directories:
            # if the name present move to the next directory
            if directory in self.__get_ftp_directories():
                self.ftp_client.cwd(directory)

        filename = self.__set_data_file_name(file_name)
        # check if the corrresponding file name present
        if filename in self.__get_ftp_directories():
            # if present write the log message to the relevant log file
            buf=io.BytesIO()
            buf.write(log_data)
            buf.seek(0)
            self.ftp_client.storbinary(f"APPE {filename}", buf, 1)

    def create_ftp_log_file(self, directory_path, file_name):
        """
        Create the log file inside an FTP server.

        Args:
            directory_name (string): The path to the created log
                                        file.
            file_name (string): The name of the log file.

        Returns:
            boolean : True of the log file was successfully created;
                        False otherwise.
        """
        if file_name == "" or file_name is None or\
            isinstance(file_name, str) is False:
            raise FTPFileCreationException()

        if directory_path == "" or directory_path is None or\
            isinstance(directory_path, str) is False:
            raise FTPFileCreationException()

        try:
            # get all the directories
            directories = directory_path.split("/")
            for directory in directories:
                # check if the directory is present
                # if not present create the directory
                # then move to the newly create directory
                if directory not in self.__get_ftp_directories():
                    self.ftp_client.mkd(directory)
                    self.ftp_client.cwd(directory)
                # if the directory present
                # just move to the newly create directory
                else:
                    self.ftp_client.cwd(directory)

            filename = self.__set_log_file_name(file_name)
            # check if the log file exist. If not create the
            # log file. otherwise, done nothing.
            if filename not in self.__get_ftp_directories():
                buf = io.BytesIO()
                buf.seek(0)
                self.ftp_client.storbinary(f"STOR {filename}", buf)
                return True
            return False
        except all_errors as error:
            raise FTPFileCreationException from error

    def set_ftp_log_file(self, directory_path, file_name, message):
        """
        Set the log file with application or system log.

        Args:
            directory_path (string): The path to the log file.
            file_name (string): The log file name.
            message (string): The message of the data.
            
         Returns:
            boolean : True of the log message was successfully written;
                        False otherwise.
        """
        if directory_path is None or file_name is None or\
            message is None:
            raise NoneValueException()

        if isinstance(directory_path, str) is False or\
            isinstance(file_name, str) is False or\
            isinstance(message, str) is False:
            raise InvalidAttributeException()

        if len(directory_path) == 0 or len(file_name) == 0 or\
            len(message) == 0:
            raise EmptyParameterException()

        # get the directories name
        directories = directory_path.split('/')
        # move to the corresponding directory if it
        # present
        for directory in directories:
            if directory in self.__get_ftp_directories():
                self.ftp_client.cwd(directory)

        # check for log file existence. If it exist only
        # write the log onto the corresponding log file.
        filename = self.__set_log_file_name(file_name)
        if filename in self.__get_ftp_directories():
            buf = io.BytesIO()
            buf.write(bytes("["+datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')+"]"+message+"\n", 'utf-8'))
            buf.seek(0)
            self.ftp_client.storbinary(f"APPE {filename}", buf, 1)



"""
The CustomLogMessage that contains the custom log message
"""
import os
import inspect
import uuid
import sys
from .myException import InvalidMessageException,\
    EmptyParameterException,\
    NoneValueException,\
    InvalidAttributeException

# pylint: disable=W0212
class CustomLog:
    """
    Class to construct the log message (info, error, debug, fatal, log data).
    """

    @staticmethod
    def set_error_message(uu_id, name, message):
        """
        Set up the format of the error message.

        Args:
            uu_id (string): The generated uuid.
            name (string): The name of the application or API.
            message (array): The error message.

        Returns:
            string: The custom error message.
        """
        if uu_id is None or name is None or message is None:
            raise NoneValueException()

        if len(str(uu_id)) != 36 or len(str(name)) == 0:
            raise EmptyParameterException()

        if isinstance(uu_id, uuid.UUID) is False or\
            isinstance(name, str) is False or\
            isinstance(message, list) is False:
            raise InvalidAttributeException()

        try:
            error_message = f"[ERROR] [{str(uu_id)}] [{str(name)}] [{os.path.basename(inspect.stack()[1].filename)}] [line {sys._getframe().f_back.f_lineno}]: {' '.join(word for word in message)}"
            return error_message
        except TypeError as exc:
            raise InvalidMessageException() from exc

    @staticmethod
    def set_debug_message(uu_id, name, message):
        """
        Set up the format of the debug message

        Args:
            uu_id (string): The generated uuid.
            name (string): The name of the application/ API/ function/ feature.
            message (list): The error message.

        Returns:
            string: The custom debug message.
        """
        if uu_id is None or name is None or message is None:
            raise NoneValueException()

        if len(str(uu_id)) != 36 or len(str(name)) == 0:
            raise EmptyParameterException()

        if isinstance(uu_id, uuid.UUID) is False or\
            isinstance(name, str) is False or\
            isinstance(message, list) is False:
            raise InvalidAttributeException()

        try:
            debug_message = f"[DEBUG] [{uu_id}] [{name}] [{os.path.basename(inspect.stack()[1].filename)}] [line {sys._getframe().f_back.f_lineno}]: {' '.join(word for word in message)}"
            return debug_message
        except TypeError as exc:
            raise InvalidMessageException() from exc

    @staticmethod
    def set_fatal_message(uu_id, name, message):
        """
        Set up the format of the fatal message.

        Args:
            uu_id (string): The generated uuid.
            name (string): The name of the application/api/function/feature.
            message (list): The error message.

        Returns:
            string: The custom fatal message.
        """
        if uu_id is None or name is None or message is None:
            raise NoneValueException()

        if len(str(uu_id)) != 36 or len(str(name)) == 0:
            raise EmptyParameterException()

        if isinstance(uu_id, uuid.UUID) is False or\
            isinstance(name, str) is False or\
            isinstance(message, list) is False:
            raise InvalidAttributeException()

        try:
            fatal_message = f"[FATAL] [{uu_id}] [{name}] [{os.path.basename(inspect.stack()[1].filename)}] [line {sys._getframe().f_back.f_lineno}]: {' '.join(word for word in message)}"
            return fatal_message
        except TypeError as exc:
            raise InvalidMessageException() from exc

    @staticmethod
    def set_info_message(uu_id, name, message):
        """
        Set up the format of the info message.

        Args:
            uu_id (string): The generated uuid.
            name (string): The name of the application or API
            message (list): The error message.

        Returns:
            string: The custom info message.
        """
        if uu_id is None or name is None or message is None:
            raise NoneValueException()

        if len(str(uu_id)) != 36 or len(str(name)) == 0:
            raise EmptyParameterException()

        if isinstance(uu_id, uuid.UUID) is False or\
            isinstance(name, str) is False or\
            isinstance(message, list) is False:
            raise InvalidAttributeException()

        try:
            info_message = f"[INFO] [{uu_id}] [{name}] [{os.path.basename(inspect.stack()[1].filename)}] [line {sys._getframe().f_back.f_lineno}]: {' '.join(word for word in message)}"
            return info_message
        except TypeError as exc:
            raise InvalidMessageException() from exc

    @staticmethod
    def set_ftp_log_data(uu_id, starttime, endtime, result, groundtruth):
        """
        Set up the log message for the FTP system.

        Args:
            uuid (string): The generated uuid.
            starttime (string): the start time of an operation.
            endtime (string): the end time of an operation.
            result (string): the result of the embedded device/ system/ model etc.
            groundtruth (string): the result of the embedded device/ system/ model etc.

        Returns:
            string: a list of log data.
        """
        if uu_id is None or starttime is None\
           or endtime is None or result is None or\
           groundtruth is None:
            raise NoneValueException()

        if isinstance(uu_id, uuid.UUID) is False or\
            isinstance(starttime, str) is False or\
            isinstance(endtime, str) is False:
            raise InvalidAttributeException()

        if len(str(uu_id)) == 0 or len(str(starttime)) == 0\
            or len(str(endtime)) == 0 or len(result) == 0\
            or len(str(groundtruth)) == 0:
            raise EmptyParameterException()

        try:
            data_message = bytes(f"{str(uuid)},{str(starttime)},\
                {str(endtime)},{float(result)},{float(groundtruth)}\n", encoding='utf-8')
            return data_message
        except ValueError as exc:
            raise InvalidMessageException() from exc



"""
The boto3 client to establish connection to the AWS cloud.
"""

import time
import boto3
from .myException import InvalidRoleAccessKeyException,\
    InvalidRoleSecretException,\
    InvalidAttributeException,\
    InvalidRegionNameException,\
    EmptyParameterException

class Boto3Client:
    """Boto 3 client to communicate to the AWS Cloudwatch.
    """

    def __init__(self, region_name, aws_key, aws_secret):
        """

        Args:
            region_name (string): The AWS service region name.
            aws_key (string): The AWS service role access key.
            aws_secret (string): The AWS service role secret key.

        Raises:
            error: When there is a communication error with AWS server.
        """
        if isinstance(region_name, str) is False or\
            isinstance(aws_key, str) is False or\
            isinstance(aws_secret, str) is False:
            raise InvalidAttributeException()

        if region_name == "" or region_name is None:
            raise InvalidRegionNameException()

        if aws_key == "" or aws_key is None:
            raise InvalidRoleAccessKeyException()

        if aws_secret == "" or aws_secret is None:
            raise InvalidRoleSecretException()

        self.client  = boto3.client("logs",
                                        region_name=region_name,
                                        aws_access_key_id=aws_key,
                                        aws_secret_access_key=aws_secret)

    def get_log_groups(self):
        """
        Get all the log groups available in the AWS cloudwatch.

        Returns:
            list: A list of log group names.
        """
        response = self.client.describe_log_groups()
        return  [logGroup['logGroupName'] for logGroup in response['logGroups']]

    def get_log_stream(self, log_group_name):
        """
        Get all the log stream inside a log group.

        Args:
            log_group_name (string): The name of the log group.

        Returns:
            list:  All the log streams' name available in a log group.
        """
        response = self.client.describe_log_streams(
            logGroupName=log_group_name
        )

        return [logGroup['logStreamName'] for logGroup in response['logStreams']]

    def __create_log_groups(self, log_group_name):
        """
        Create the log group if the not group is not present.

        Args:
            log_group_name (string): the log group name that wanted to be created.

        Returns:
            bool: True if the operation to create the log group was a success or False
                  otherwise.
        """
        if log_group_name not in self.get_log_groups():
            response = self.client.create_log_group(
                logGroupName=log_group_name,
            )
            return response
        return None

    def create_log_group_stream(self, log_group_name, log_stream_name):
        """
        Create the log stream if the current log group is present. Otherwise,
        create a new log group then create the desire the log stream

        Args:
            log_group_name (string): the log group name.
            log_stream_name (string): the log stream name.

        Returns:
            bool: whether the operation is a success (True) or otherwise (False).
        """
        if isinstance(log_group_name, str) is False or\
            isinstance(log_stream_name, str) is False:
            raise InvalidAttributeException()

        if log_stream_name == "" or log_group_name == "":
            raise EmptyParameterException()

        # check if the relevant log stream or log group
        # is present
        if log_group_name in self.get_log_groups() and \
          log_stream_name not in self.get_log_stream(log_group_name):
          # if the log stream is not present
          # create the log stream.
            self.client.create_log_stream(
              logGroupName=log_group_name,
              logStreamName=log_stream_name
            )
            return True

        # If the log group is not present, create the log group
        log_group_response = self.__create_log_groups(log_group_name)
        # If the log group is successfully created
        if log_group_response is not None:
            # create the log stream and return the response
            self.create_log_group_stream(log_group_name, log_stream_name)
            return True

        # Otherwise, return None to show the log stream and log group was present
        return False

    def set_log_message(self, log_group_name, log_stream_name, log_message):
        """
        Allow the application to push the relevant log message to the cloudwatch.

        Args:
            log_group_name (string): The name of the log group.
            log_stream_name (string): The name of the log stream.
            log_message (list): A list of application/ service/ api message.

        Returns:
            response: A list of aws responses.
        """
        if isinstance(log_group_name, str) is False or\
            isinstance(log_stream_name, str) is False or\
            isinstance(log_message, str) is False:
            raise InvalidAttributeException()

        if log_group_name == "" or log_stream_name == "" or\
            log_message == "":
            raise EmptyParameterException()

        # check if the log_group name and log stream is present
        if log_group_name in self.get_log_groups() and \
            log_stream_name in self.get_log_stream(log_group_name):
            # if present only start to push the log message
            # onto the cloudwatch.
            self.client.put_log_events(
                logGroupName=log_group_name,
                logStreamName=log_stream_name,
                logEvents=[
                    {
                        'timestamp': int(round(time.time() * 1000)),
                        'message': time.strftime('%Y-%m-%d %H:%M:%S') + \
                            "\t" + log_message
                    }
                ]
            )
            return True
        return False



import os
from setuptools import setup, find_packages

BUILD_ID = os.environ.get("BUILD_BUILDID", "0")

setup(
    name="loris_log",
    version="0.1" + "." + BUILD_ID,
    # Author details
    author='Kah Seng, Sze Ling, Stanley',
    author_email='lim0709@gmail.com, sltang@handalindah.com.my, stanly@handalindah.com.my',
    packages=['loris_log']
    # packages=find_packages(""),
    
)


"""
The example of how to use this library.
"""


import sys 
import os
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))
import uuid
import datetime
from datetime import timedelta
from loris_log.boto3Client import Boto3Client
from loris_log.customLog import CustomLog
from loris_log.ftpCllient import FtpClient

def get_log_group_sample():
    """
    The example of getting the log group names.
    """
    target_region_name = ""
    target_aws_key = ""
    target_aws_secret = ""
    boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
    response = boto3Client.get_log_groups()
    print(response)
    
def create_log_group_sample():
    """
    The example of creating the log group.
    """
    target_log_group = "sample"
    target_log_stream = "sample"
    target_region_name = ""
    target_aws_key = ""
    target_aws_secret = ""
    boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
    response = boto3Client.create_log_group_stream(target_log_group, target_log_stream)
    print(response, " is true")
    
def get_log_stream_sample():
    """
    The example of creating the log stream.
    """
    target_log_group = "sample"
    target_region_name = ""
    target_aws_key = ""
    target_aws_secret = ""
    boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
    response = boto3Client.get_log_stream(target_log_group)
    print("response: ", response)
    
def set_custom_message_sample():
    """
    The example of setting various log message.

    Returns:
        string: various log mesasges.
    """
    id = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
    target_name = "my_application_name"
    target_error_message = ["[404]", "File not found exception."]
    target_debug_message = ["[200]", "Running the time calculation"]
    target_fatal_message = ["[500]", "System Halt"]
    target_info_message = ["[200]", "model accuracy", "0.99"]

    error_message = CustomLog.set_error_message(id, target_name, target_error_message)
    debug_message = CustomLog.set_debug_message(id, target_name, target_debug_message)
    fatal_message = CustomLog.set_fatal_message(id, target_name, target_fatal_message)
    info_message = CustomLog.set_info_message(id, target_name, target_info_message)
    return error_message, debug_message, fatal_message, info_message
    
def set_log_message_sample():
    """
    The example of how to pushing log message onto the cloudwatch.
    """
    target_log_group = "sample"
    target_log_stream = "sample-log"
    target_region_name = ""
    target_aws_key = ""
    target_aws_secret = ""
    boto3Client = Boto3Client(target_region_name, target_aws_key, target_aws_secret)
    response = create_log_group_sample()
    if response is None:
        error_message, debug_message, fatal_message, info_message = set_custom_message_sample()
        error_log_response = boto3Client.set_log_message(target_log_group, target_log_stream, error_message)
        debug_log_response = boto3Client.set_log_message(target_log_group, target_log_stream, debug_message)
        fatal_log_response = boto3Client.set_log_message(target_log_group, target_log_stream, fatal_message)
        info_log_response = boto3Client.set_log_message(target_log_group, target_log_stream, info_message)
        if error_log_response is not None:
            print("log inserted successfully.")
            
            
def set_log_data_ftp_example():
    """Example of how to log data onto the csv log file.
    """
    id = uuid.uuid4()
    target_hostname = ""
    target_port = 21
    target_username = ""
    target_password = ""
    ftp_client = FtpClient(target_hostname, target_port, target_username, target_password)
    ftp_client.create_ftp_log_data("log_testing/data", "id")
        
    start_time = datetime.datetime.now()
    target_start_time = start_time.strftime("%H:%M:%S")
    end_time = datetime.datetime.now()
    target_end_time = end_time.strftime("%H:%M:%S")
    target_result = "32"
    target_ground_truth = "3"
    file_data_message = CustomLog.set_ftp_log_data(id,
                                    target_start_time, 
                                    target_end_time, 
                                    target_result, 
                                    target_ground_truth)
    for i in range(5):
        ftp_client.set_ftp_log_data("log_testing/data", "id", file_data_message)
        
def set_log_file_ftp_example():
    """Example of how to log logging message to the log file
    """
    id = uuid.uuid4()
    target_hostname = ""
    target_port = 21
    target_username = ""
    target_password = ""
    print(id)
    ftp_client = FtpClient(target_hostname, target_port, target_username, target_password)
    response = ftp_client.create_ftp_log_file("log_testing/data", "id")
      
    target_name = "sample log"
    target_info_message = ["model accuracy", "0.99"]
    info_message = CustomLog.set_info_message(id, target_name, target_info_message)
    for i in range(5):
        ftp_client.set_ftp_log_file("log/testing/data", "id", info_message)
            

def main():
    set_log_message_sample()
    # get_event_log_example()
    set_log_data_ftp_example()
    set_log_file_ftp_example()

if __name__=="__main__":
    main()